@@ -1,8 +1,9 @@
-from abc import abstractmethod
-import numpy
-
 from ..interface import Contract, ContractNotRespected, RValue, eval_in_context
-from ..syntax import W
+from ..syntax import W, Keyword, add_contract, add_keyword
+from .types_misc import CheckType
+from abc import abstractmethod
+import numpy as np
+
 
 
 class ArrayElementsTest(Contract):
@@ -13,12 +14,12 @@ class ArrayElementsTest(Contract):
 
     def check_contract(self, context, value):
         result = self.test_elements(context, value)
-        if numpy.all(result):
+        if np.all(result):
             return
-        result = numpy.array(result)  # for simple bool
+        result = np.array(result)  # for simple bool
         resultf = result.flatten()
         valuef = value.flatten()
-        some, = numpy.nonzero(numpy.logical_not(resultf))
+        some, = np.nonzero(np.logical_not(resultf))
         num = value.size
         num_fail = len(some)
         perc = 100.0 * num_fail / num
@@ -61,11 +62,11 @@ class ArrayOR(ArrayLogical):
         self.clauses = clauses
 
     def test_elements(self, context, value):
-        assert isinstance(value, numpy.ndarray)
+        assert isinstance(value, np.ndarray)
         result = False
         for c in self.clauses:
             result_c = c.test_elements(context, value)
-            result = numpy.logical_or(result_c, result)
+            result = np.logical_or(result_c, result)
         return result
 
     def __repr__(self):
@@ -84,6 +85,14 @@ class ArrayOR(ArrayLogical):
         where = W(string, location)
         return ArrayOR(clauses, where=where)
 
+class ArrayORCustomString(ArrayOR):
+    def __init__(self, custom_string, **other):
+        self.custom_string = custom_string
+        ArrayOR.__init__(self, **other)
+        
+    def __str__(self):
+        return self.custom_string
+
 
 class ArrayAnd(ArrayLogical):
     def __init__(self, clauses, where=None):
@@ -99,11 +108,11 @@ class ArrayAnd(ArrayLogical):
         self.clauses = clauses
 
     def test_elements(self, context, value):
-        assert isinstance(value, numpy.ndarray)
+        assert isinstance(value, np.ndarray)
         result = True
         for c in self.clauses:
             result_c = c.test_elements(context, value)
-            result = numpy.logical_and(result_c, result)
+            result = np.logical_and(result_c, result)
         return result
 
     def __repr__(self):
@@ -146,7 +155,7 @@ class ArrayConstraint(ArrayElementsTest):
 
     def test_elements(self, context, value):
         ''' Returns either a bool or an array of bool. '''
-        assert isinstance(value, numpy.ndarray)
+        assert isinstance(value, np.ndarray)
         bound = eval_in_context(context=context, value=self.rvalue,
                                 contract=self)
 
@@ -171,7 +180,7 @@ class ArrayConstraint(ArrayElementsTest):
 class DType(ArrayElementsTest):
     ''' Checks that the value is an array with the given dtype. '''
     def __init__(self, dtype, dtype_string=None, where=None):
-        assert isinstance(dtype, numpy.dtype)
+        assert isinstance(dtype, np.dtype)
         Contract.__init__(self, where)
         self.dtype = dtype
         if dtype_string is None:
@@ -179,7 +188,7 @@ class DType(ArrayElementsTest):
         self.dtype_string = dtype_string
 
     def test_elements(self, context, value):  # @UnusedVariable
-        assert isinstance(value, numpy.ndarray)  # Guaranteed by construction
+        assert isinstance(value, np.ndarray)  # Guaranteed by construction
         return (value.dtype == self.dtype)
 
     def __str__(self):
@@ -193,16 +202,41 @@ class DType(ArrayElementsTest):
 
     @staticmethod
     def parse_action(dtype=None):
-        assert dtype is None or isinstance(dtype, numpy.dtype)
+        assert dtype is None or isinstance(dtype, np.dtype)
 
         def parse(s, loc, tokens):
             where = W(s, loc)
             dtype_string = tokens[0]
             if dtype is None:
-                use_dtype = numpy.dtype(dtype_string)
+                use_dtype = np.dtype(dtype_string)
             else:
                 use_dtype = dtype
             return DType(use_dtype, dtype_string, where)
         return parse
 
 
+np_types = {
+    'np_int': np.int,  # Platform integer (normally either int32 or int64)
+    'np_int8': np.int8,  # Byte (-128 to 127)
+    'np_int16': np.int16,  # Integer (-32768 to 32767)
+    'np_int32': np.int32,  # Integer (-2147483648 to 2147483647)
+    'np_int64': np.int64,  # Integer (9223372036854775808 to 9223372036854775807)
+    'np_uint8': np.uint8,  # Unsigned integer (0 to 255)
+    'np_uint16': np.uint16,  # Unsigned integer (0 to 65535)
+    'np_uint32': np.uint32,  # Unsigned integer (0 to 4294967295)
+    'np_uint64': np.uint64,  # Unsigned integer (0 to 18446744073709551615)
+    'np_float': np.float,  # Shorthand for float64.
+    'np_float16': np.float16,  #  Half precision float: sign bit, 5 bits exponent, 10 bits mantissa
+    'np_float32': np.float32,  #  Single precision float: sign bit, 8 bits exponent, 23 bits mantissa
+    'np_float64': np.float64,  #  Double precision float: sign bit, 11 bits exponent, 52 bits mantissa
+    'np_complex': np.complex,  #  Shorthand for complex128.
+    'np_complex64': np.complex64,  #    Complex number, represented by two 32-bit floats (real and imaginary components)
+    'np_complex128': np.complex128}
+
+for k, t in np_types.items():
+    add_contract(Keyword(k).setParseAction(CheckType.parse_action(t)))
+    add_keyword(k)
+
+
+
+
