@@ -1,18 +1,16 @@
 from .backported import getcallargs, getfullargspec
-from .docstring_parsing import DocStringInfo, Arg
+from .docstring_parsing import Arg, DocStringInfo
 from .enabling import all_disabled
-from .interface import (Contract, ContractSyntaxError, Where,
-                        ContractException, ContractNotRespected,
-                        describe_value, MissingContract)
-from .library import (identifier_expression, Extension, CheckCallable,
-                      SeparateContext)
+from .interface import (CannotDecorateClassmethods, Contract, ContractException, 
+    ContractNotRespected, ContractSyntaxError, MissingContract, Where, 
+    describe_value)
+from .library import (CheckCallable, Extension, SeparateContext, 
+    identifier_expression)
 from .library.extensions import CheckCallableWithSelf
-from .syntax import contract_expression, ParseException, ParseFatalException
+from .syntax import ParseException, ParseFatalException, contract_expression
 import inspect
 import sys
 import types
-from contracts.interface import ContractDefinitionError, \
-    CannotDecorateClassmethods
 
 
 def check_contracts(contracts, values, context_variables=None):
@@ -270,11 +268,13 @@ you can achieve the same goal by inverting the two decorators:
         if not do_checks:
             return function_(*args, **kwargs)
 
-        nice_function_display = '%s()' % function_.__name__
-        if is_bound_method:
-            klass = type(args[0]).__name__
-            nice_function_display = klass + ':' + nice_function_display  
-
+        def get_nice_function_display():
+            nice_function_display = '%s()' % function_.__name__
+            if is_bound_method:
+                klass = type(args[0]).__name__
+                nice_function_display = klass + ':' + nice_function_display  
+            return nice_function_display
+        
         bound = getcallargs(function_, *args, **kwargs)
 
         try:
@@ -289,7 +289,7 @@ you can achieve the same goal by inverting the two decorators:
 
         except ContractNotRespected as e:
             msg = ('Breach for argument %r to %s.\n'
-                   % (arg, nice_function_display))
+                   % (arg, get_nice_function_display()))
             e.error = msg + e.error
             raise e
 
@@ -300,7 +300,7 @@ you can achieve the same goal by inverting the two decorators:
                 returns_parsed._check_contract(context, result)
             except ContractNotRespected as e:
                 msg = ('Breach for return value of %s.\n'
-                       % (nice_function_display))
+                       % (get_nice_function_display()))
                 e.error = msg + e.error
                 raise e
 
